// PoC Code by Gynvael Coldwind
// http://gynvael.coldwind.pl
// Very Simple OpenGL daemon for batch ;>
// g++ GLdaemon.cpp NetSock.cpp -lopengl32 -lGLU32 -lSDL -lws2_32 -o GLDaemon -mwindows

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <gl/gl.h>
#include <gl/glu.h>
#include <SDL/sdl.h>
#ifdef _WIN32
 #include <windows.h>
#endif
#include <list>
#include "NetSock.h"

using namespace std;

list<char*> CmdList;

bool Connection;
bool UsingList;
bool UserLock;

// SDL sugz
#undef main

// globals
static int screen_width;
static int screen_height;
bool keys[SDLK_LAST];

// func
static bool initSDL(bool fullscreen = false, int width = 640, int height = 480);
static bool initOpenGL();
static void scene();
static bool events();

void SocketThread(void *nothing)
{
  (void)nothing; // UNUSED
  NetSock ls;

  printf("Listening: %i\n", ls.Listen(31337));

  while(1)
  {
    NetSock *ac = ls.Accept();

    if(!ac)
      continue;

    Connection = true;

    // Read data
    char Buffer[1024];
    memset(Buffer, 0, 1024);
    int ret = ac->Read((unsigned char*)Buffer, 1024);
    ac->Disconnect();
    delete ac;

    if(ret <= 0)
      continue;

    while(UsingList) Sleep(0);

    // Check command
    if(Buffer[0] == 'C') // Clear list
    {
      puts("Clear");
      list<char*>::iterator i;

      for(i = CmdList.begin(); i != CmdList.end(); i++)
        free(*i);

      CmdList.clear();
    }
    else if(Buffer[0] == 'A') // Add to list
    {
      printf("Adding cmd: %s\n", Buffer+1);
      CmdList.push_back(strdup(Buffer+1));
    }
    else if(Buffer[0] == 'L') // Lock
    {
      puts("Lock");
      UserLock = true;
    }
    else if(Buffer[0] == 'U') // Unlock
    {
      puts("Unlock");
      UserLock = false;
    }

    // No more connection
    Connection = false; 
  }
}

// main
int 
main()
{
  DWORD a;
  CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)SocketThread, NULL, 0, &a);

  if(!initSDL())    return 1;
  if(!initOpenGL()) return 2;

  while(events()) scene();


  SDL_Quit();
  ExitProcess(0);

  return 0;
}

// scene
void
static scene()
{
  if(Connection || UserLock)
    return;

  UsingList = true;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  list<char*>::iterator i;

  for(i = CmdList.begin(); i != CmdList.end(); i++)
  {
    char Cmd[256];
    sscanf(*i, "%256s", Cmd);

    //printf("Issuing cmd: %s (%s)\n", Cmd, *i);

    if(strcmp(Cmd, "glTranslatef") == 0)
    {
      float a, b, c;
      sscanf(*i, "%*s %f %f %f", &a, &b, &c);
      glTranslatef(a,b,c);
    }
    else if(strcmp(Cmd, "glRotatef") == 0)
    {
      float a, b, c, d;
      sscanf(*i, "%*s %f %f %f %f", &a, &b, &c, &d);
      glRotatef(a,b,c,d);
    }
    else if(strcmp(Cmd, "glBegin") == 0)
    {
      char What[32];
      sscanf(*i, "%*s %s", What);

      if(strcmp(What, "GL_TRIANGLES") == 0)
        glBegin(GL_TRIANGLES);
    }
    else if(strcmp(Cmd, "glColor3f") == 0)
    {
      float a, b, c;
      sscanf(*i, "%*s %f %f %f", &a, &b, &c);
      glColor3f(a,b,c);
    }
    else if(strcmp(Cmd, "glVertex3f") == 0)
    {
      float a, b, c;
      sscanf(*i, "%*s %f %f %f", &a, &b, &c);
      glVertex3f(a,b,c);
    }
    else if(strcmp(Cmd, "glEnd") == 0)
    {
      glEnd();     
    }
    else if(strcmp(Cmd, "glPushMatrix") == 0)
    {
      glPushMatrix();
    }
    else if(strcmp(Cmd, "glPopMatrix") == 0)
    {
      glPopMatrix();
    }
  }

  UsingList = false;

  SDL_GL_SwapBuffers(); 
}

// events
static bool
events()
{
  SDL_Event ev;
  memset(&ev, 0, sizeof(ev));
  while(SDL_PollEvent(&ev))
  {
    switch( ev.type )
    { 
      case SDL_KEYUP:
	keys[ev.key.keysym.sym] = false;	
	break;
      
      
      case SDL_KEYDOWN:
        if(ev.key.keysym.sym == SDLK_ESCAPE)
          return false;
	keys[ev.key.keysym.sym] = true;
	break;	
	
      case SDL_QUIT:
        return false;      
    }
  }
  
  return true;
}

// initSDL
static bool
initSDL(bool fullscreen, int width, int height)
{
  int screen_bpp;
  int screen_flag = SDL_OPENGL;
  if(fullscreen) screen_flag |= SDL_FULLSCREEN;

  screen_width = width;
  screen_height = height;
 
  const SDL_VideoInfo *info;
  SDL_Surface *surface;  

  if( SDL_Init(SDL_INIT_VIDEO) < 0 )
    return false;
 
  info = SDL_GetVideoInfo( );
  if(info == NULL)
    goto err;

  screen_bpp = info->vfmt->BitsPerPixel;
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  surface = SDL_SetVideoMode(
            screen_width, screen_height, 
            screen_bpp, screen_flag);

  if(surface == NULL)
    goto err;

  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

  return true;
  
err:
  SDL_Quit();
  return false;
}

// initOpenGL
static bool
initOpenGL()
{
  float ratio;
  ratio = (float)screen_width / (float)screen_height;
  
  glShadeModel(GL_SMOOTH);  
  glClearColor(0, 0, 0, 0);  
  glViewport(0, 0, screen_width, screen_height);  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity( );
  gluPerspective( 60.0, ratio, 1.0, 1024.0 );
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  // glDisable(GL_DEPTH_TEST); // uncomment this if going 2D
  return true;
}

