// Consider this to be on a BSD style license
// For licensing details contant me at gynvael@coldwind.pl
// http://gynvael.coldwind.pl
#include <windows.h>
#include <winsock.h>
#include <cstdlib>
#include "NetSock.h"

const int NetSock::SYNCHRONIC  = 1;
const int NetSock::ASYNCHRONIC = 2;
bool NetSock::WinSockInited;

NetSock::NetSock()
{
  this->socket = -1;
  this->ip     = 0x00000000;
  this->mode   = 0;
  this->port   = 0;

  this->str_ip[0] = '\0';

  if(!NetSock::WinSockInited)
  {
    NetSock::WinSockInited = true;
    WSADATA wsdat;
    memset(&wsdat,0,sizeof(wsdat));
    WSAStartup(0x0101,&wsdat);
  }
}

NetSock::~NetSock()
{
  this->Disconnect();
}

bool
NetSock::Listen(unsigned short port)
{
 sockaddr_in desc;
 int ret;

  ret = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if(ret == -1)
  {
    puts("sock");
    return false;
  }

  this->socket = ret;

  desc.sin_family = AF_INET;
  desc.sin_addr.s_addr = inet_addr("0.0.0.0");
  desc.sin_port = htons(port);
  if(bind(this->socket, (sockaddr*)&desc, sizeof(sockaddr)) == -1)
  {
    puts("bind");
    return false;
  }

  if(listen(this->socket, SOMAXCONN) == -1)
  {
    puts("listen");
    return false;
  }

  return true;
}

NetSock *
NetSock::Accept()
{
 sockaddr_in desc;
  int remote;
  int size;

  size = sizeof(sockaddr);
  remote = accept(this->socket, (sockaddr*)&desc, &size);
  if(remote == -1)
    return NULL;

  NetSock *NewSock = new NetSock;

  NewSock->socket = remote;
  NewSock->port   = htons(desc.sin_port);
  memcpy(&NewSock->ip, &desc.sin_addr.s_addr, 4);

  return NewSock;
}

bool
NetSock::Connect(const char* host, unsigned short port)
{
  unsigned int ip;

  ip = (unsigned int)inet_addr(host);
  if(ip == INADDR_NONE)
  {
    struct hostent *hostip;

    // resolve
    hostip = gethostbyname(host);
    if(!hostip)
      return false;

    memcpy(&ip, hostip->h_addr_list[0], 4);
  }

  return this->Connect(ip, port);
}

bool 
NetSock::Connect(unsigned int ip, unsigned short port)
{
  struct sockaddr_in sock_info;
  int ret, sock = -1;

  if(this->socket != -1)
    throw "Socket already exists";

  ret = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if(ret == -1)
    throw "Socket create failed";

  sock = ret;

  // sock info
  sock_info.sin_family = AF_INET;
  sock_info.sin_addr.s_addr = htonl(ip);
  sock_info.sin_port = htons(port);
  memset(sock_info.sin_zero, 0, sizeof(sock_info.sin_zero));

  // conn
  ret = connect(sock, (struct sockaddr*)&sock_info, sizeof( struct sockaddr ));
  if(ret == -1)
  {
    closesocket(sock);
    return false;
  }

  // ok
  this->ip = ip;
  this->port = port;
  this->mode = this->SYNCHRONIC;
  this->socket = sock;
  return true;
}
  
bool 
NetSock::SetMode(int mode)
{
  unsigned long mode_temp;
  if(this->socket == -1)
    return false;
  
  if(this->mode == mode)
    return true;

  switch(mode)
  {
    case NetSock::SYNCHRONIC:
      mode_temp = 0;
      ioctlsocket(this->socket, FIONBIO, &mode_temp);
      break;

    case NetSock::ASYNCHRONIC:
      mode_temp = 1;
      ioctlsocket(this->socket, FIONBIO, &mode_temp);
      break;
  }

  this->mode = mode;
  return true;
}

bool 
NetSock::Disconnect()
{
  if(this->socket == -1)
    return false;

  closesocket(this->socket);
  this->socket = -1;
  this->ip     = 0x00000000;
  this->mode   = 0;
  this->port   = 0;
  return true;
}

int 
NetSock::Read(unsigned char *Buffer, int Size)
{
  if(this->socket == -1)
    return -1;

  return recv(this->socket, (char*)Buffer, Size, 0);
}

int 
NetSock::Write(unsigned char *Buffer, int Size)
{
  if(this->socket == -1)
    return -1;

  return send(this->socket, (char*)Buffer, Size, 0);
}

unsigned short 
NetSock::GetPort() const
{
  return this->port;
}

unsigned int
NetSock::GetIP() const
{
  return this->ip;
}

const char *
NetSock::GetStrIP()
{
  // cached ?
  if(this->str_ip[0])
    return this->str_ip;

  strncpy(this->str_ip, inet_ntoa(*(in_addr*)&this->ip), 16);
  this->str_ip[15] = '\0';
  return this->str_ip;
}

